@@ -99,6 +99,16 @@ window.utilities = {
     return window.innerHeight ||
            document.documentElement.clientHeight ||
            document.body.clientHeight;
+  },
+
+  /**
+   * Return the offset amount to deduct from the normal scroll position.
+   * Modify as appropriate to allow for dynamic calculations
+   */
+  getFixedOffset: function() {
+    var OFFSET_HEIGHT_PADDING = 0;
+    // TODO: this is a little janky. We should try to not rely on JS for this
+    return document.getElementById("sphinx-template-page-level-bar").offsetHeight + OFFSET_HEIGHT_PADDING;
   }
 }
 
@@ -150,7 +160,7 @@ $(function() {
 // Modified from https://stackoverflow.com/a/32396543
 window.highlightNavigation = {
   navigationListItems: document.querySelectorAll("#sphinx-template-right-menu li"),
-  sections: document.querySelectorAll(".sphinx-template-article .section"),
+  sections: document.querySelectorAll(".sphinx-template-article section section, .sig.sig-object"),
   sectionIdTonavigationLink: {},
 
   bind: function() {
@@ -329,30 +339,17 @@ window.scrollToAnchor = {
 
     var anchorScrolls = {
       ANCHOR_REGEX: /^#[^ ]+$/,
-      offsetHeightPx: function() {
-        var OFFSET_HEIGHT_PADDING = 20;
-        // TODO: this is a little janky. We should try to not rely on JS for this
-        return utilities.headersHeight() + OFFSET_HEIGHT_PADDING;
-      },
-
       /**
        * Establish events, and fix initial scroll position if a hash is provided.
        */
       init: function() {
         this.scrollToCurrent();
         // This interferes with clicks below it, causing a double fire
-        // $(window).on('hashchange', $.proxy(this, 'scrollToCurrent'));
+        $(window).on('hashchange', $.proxy(this, 'scrollToCurrent'));
         $('body').on('click', 'a', $.proxy(this, 'delegateAnchors'));
         $('body').on('click', '#sphinx-template-right-menu li span', $.proxy(this, 'delegateSpans'));
       },
 
-      /**
-       * Return the offset amount to deduct from the normal scroll position.
-       * Modify as appropriate to allow for dynamic calculations
-       */
-      getFixedOffset: function() {
-        return this.offsetHeightPx();
-      },
 
       /**
        * If the provided href is an anchor which resolves to an element on the
@@ -370,7 +367,7 @@ window.scrollToAnchor = {
         match = document.getElementById(href.slice(1));
 
         if(match) {
-          var anchorOffset = $(match).offset().top - this.getFixedOffset();
+          var anchorOffset = $(match).offset().top - utilities.getFixedOffset();
 
           $('html, body').scrollTop(anchorOffset);
 
@@ -425,8 +422,6 @@ window.sideMenus = {
   isFixedToBottom: false,
 
   bind: function() {
-    sideMenus.handleLeftMenu();
-
     var rightMenuLinks = document.querySelectorAll("#sphinx-template-right-menu li");
     var rightMenuHasLinks = rightMenuLinks.length > 1;
 
@@ -464,18 +459,30 @@ window.sideMenus = {
       }
 
       // Add + expansion signifiers to normal right menu links that have sub menus
-      var menuLinks = document.querySelectorAll(
-        "#sphinx-template-right-menu ul li ul li a.reference.internal"
-      );
-
-      for (var i = 0; i < menuLinks.length; i++) {
+      $('#sphinx-template-right-menu ul li ul li a.reference.internal').each(function () {
         if (
-          menuLinks[i].nextElementSibling &&
-          menuLinks[i].nextElementSibling.tagName === "UL"
+          this.nextElementSibling &&
+          this.nextElementSibling.tagName === "UL"
         ) {
-          menuLinks[i].classList.add("not-expanded");
+          var link = $(this)
+          var next = this.nextElementSibling
+          link.attr('aria-expanded', 'false');
+          var expand = $('<button class="toctree-expand" title="Open/close menu"></button>');
+          expand.on('click', function (ev) {
+            if (link.attr('aria-expanded')==='true') {
+              next.style.display = 'none';
+              link.attr('aria-expanded', 'false')
+            }
+            else {
+              next.style.display = 'block';
+              link.attr('aria-expanded', 'true')
+            }
+            ev.stopPropagation();
+            return false;
+          });
+          link.prepend(expand);
         }
-      }
+      });
 
       // If a hash is present on page load recursively expand menu items leading to selected item
       var linkWithHash =
@@ -491,28 +498,18 @@ window.sideMenus = {
           linkWithHash.nextElementSibling.children.length > 0
         ) {
           linkWithHash.nextElementSibling.style.display = "block";
-          linkWithHash.classList.remove("not-expanded");
-          linkWithHash.classList.add("expanded");
+          $(linkWithHash).attr('aria-expanded', 'true');
         }
 
         // Expand ancestor lists if any
         sideMenus.expandClosestUnexpandedParentList(linkWithHash);
       }
 
-      // Bind click events on right menu links
-      $('#sphinx-template-right-menu').on('click', 'a.reference.internal', function() {
-        if (this.classList.contains("expanded")) {
-          this.nextElementSibling.style.display = "none";
-          this.classList.remove("expanded");
-          this.classList.add("not-expanded");
-        } else if (this.classList.contains("not-expanded")) {
-          this.nextElementSibling.style.display = "block";
-          this.classList.remove("not-expanded");
-          this.classList.add("expanded");
-        }
-      });
-
-      sideMenus.handleRightMenu();
+      sideMenus.handleNavBar();
+      sideMenus.handleLeftMenu();
+      if (sideMenus.rightMenuIsOnScreen()) {
+        sideMenus.handleRightMenu();
+      }
     }
 
     $(window).on('resize scroll', function(e) {
@@ -559,8 +556,7 @@ window.sideMenus = {
          }
 
         closestParentList.style.display = "block";
-        closestParentLink.classList.remove("not-expanded");
-        closestParentLink.classList.add("expanded");
+        $(closestParentLink).attr('aria-expanded', 'true');
         sideMenus.expandClosestUnexpandedParentList(closestParentLink);
       }
     }
@@ -766,7 +762,7 @@ function ThemeNav () {
                 // Find associated id element, then its closest section
                 // in the document and try with that one.
                 var id_elt = $('.document [id="' + anchor.substring(1) + '"]');
-                var closest_section = id_elt.closest('div.section');
+                var closest_section = id_elt.closest('section section, .sig.sig-object');
                 link = vmenu.find('[href="#' + closest_section.attr("id") + '"]');
                 if (link.length === 0) {
                     // still not found in the sidebar. fall back to main section
@@ -790,7 +786,6 @@ function ThemeNav () {
                         .addClass('current')
                         .attr('aria-expanded','true');
                 }
-                link[0].scrollIntoView();
             }
             $('.sphinx-template-menu-vertical li.toctree-l1')
                 .attr('aria-expanded','true');
@@ -978,42 +973,93 @@ $(".stars-outer > i").on("click", function() {
 })
 
 $("#sphinx-template-side-scroll-right").on("click", "a.reference.internal", function (e) {
-  var href = $(this).attr("href").replaceAll('.', '\\.');
-  $('html, body').stop().animate({
-    scrollTop: $(href).offset().top - 100
-  }, 850);
-  e.preventDefault;
+  var link = $(this)
+  var href = link.attr("href").replaceAll('.', '\\.');
+  var offset = 0
+  if (href !== "#"){
+    offset = $(href).offset().top - utilities.getFixedOffset()
+  }
+  $('html').stop().animate({scrollTop: offset}, 850, function (){
+    link.children("button").trigger("click");
+  });
+  e.preventDefault();
+  e.stopPropagation();
 });
 
-var lastId,
-  topMenu = $("#sphinx-template-side-scroll-right"),
-  topMenuHeight = topMenu.outerHeight() + 1,
-  // All sidenav items
-  menuItems = topMenu.find("a"),
-  // Anchors for menu items
-  scrollItems = menuItems.map(function () {
-    var item = $(this).attr("href");
-    if (item.length) {
-      return item;
-    }
+topMenu = $("#sphinx-template-side-scroll-right"),
+// All sidenav items
+menuItems = topMenu.find("a[href^='#']"),
+// Anchors for menu items
+scrollItems = {};
+for (var i = 0; i < menuItems.length; i++) {
+  var ref = menuItems[i].getAttribute("href").replaceAll('.', '\\.');
+  if (ref.length > 1 && $(ref).length) {
+    scrollItems[ref] = menuItems[i];
+  }
+}
+
+findParent = function(item) {
+  return $(item).parent().parent().siblings("a.reference.internal")
+},
+makeHighlight = function(item) {
+  if ($(item).hasClass("title-link")) {
+    return
+  }
+  $(item).addClass("side-scroll-highlight");
+  var parent = findParent(item);
+  if (parent.length) {
+    makeHighlight(parent)
+  }
+},
+showHighlight = function(item) {
+  $(menuItems).removeClass("side-scroll-highlight");
+  $(menuItems).removeClass("current");
+  $(item).addClass("current")
+  makeHighlight(item);
+  $("#sphinx-template-right-menu ul li ul li a.reference.internal[aria-expanded='true']").each(function () {
+    this.nextElementSibling.style.display = 'none';
+    $(this).attr('aria-expanded', 'false');
   });
+  sideMenus.expandClosestUnexpandedParentList(item);
+},
+initHighlight = function() {
+  item_list = $(Object.keys(scrollItems).join(', '))
+  if (item_list.length) {
+    var value = -1e10;
+    var idx = -1;
+    for (var i = 0; i < item_list.length; i++) {
+      var offset = $(item_list[i]).offset().top - $(window).scrollTop() - utilities.getFixedOffset();
+      if (offset <= 50 && offset > value) {
+        value = offset;
+        idx = i;
+      }
+    }
+    if (idx !== -1) {
+      showHighlight(scrollItems['#' + item_list[idx].id.replaceAll('.', '\\.')])
+    }
+  }
+};
 
-$(window).scroll(function () {
-  var fromTop = $(this).scrollTop() + topMenuHeight;
-  var article = ".section";
-
-  $(article).each(function (i) {
-    var offsetScroll = $(this).offset().top - $(window).scrollTop();
-    if (
-      offsetScroll <= topMenuHeight + 200 &&
-      offsetScroll >= topMenuHeight - 200 &&
-      scrollItems[i] == "#" + $(this).attr("id") &&
-      $(".hidden:visible")
-    ) {
-      $(menuItems).removeClass("side-scroll-highlight");
-      $(menuItems[i]).addClass("side-scroll-highlight");
+$(window).scroll(function() {
+  $(Object.keys(scrollItems).join(', ')).each(function () {
+    var offsetScroll = $(this).offset().top - $(window).scrollTop() - utilities.getFixedOffset();
+    if (offsetScroll <= 30 && offsetScroll >= -30) {
+      var menu_item = scrollItems['#' + this.id.replaceAll('.', '\\.')]
+      if (!$(menu_item).hasClass("side-scroll-highlight")){
+        showHighlight(menu_item)
+      }
+      else if (!$(menu_item).hasClass("current")) {
+        $(menuItems).removeClass("current");
+        $(menu_item).addClass("current")
+        var item_list = menu_item.nextElementSibling
+        $(item_list).find("a.reference.internal").removeClass("side-scroll-highlight");
+        $(item_list).find("ul").attr('display', 'none').attr('aria-expanded', 'false');
+      }
     }
   });
 });
 
+$(document).ready(initHighlight);
+$(window).on('hashchange', initHighlight);
+
 },{"jquery":"jquery"}]},{},[1,2,3,4,5,6,7,8,9,"sphinx-template-sphinx-theme"]);
